---
title:  "Say O(n²) Eighty Eight Times Fast"
comment_links:
  reddit: https://www.reddit.com/r/programming/comments/2hth4h/say_on%C2%B2_eighty_eight_times_fast/
  hacker news: https://news.ycombinator.com/item?id=8384945
---

The symbol `<` is spoken "less than", but how did you know that? Unfortunately, it's common practice for documentation and educational material to teach words and symbols without an adequate explanation of how to say them. And often, googling it doesn't work.

Why does it matter? Of course for communicating, but also for your own skill and understanding. When you read, you still think the word verbally, so you would end up choosing a word like "left angle bracket" which is less intuitive, and probably leaves a lingering sense of uncertainty about it. Yuck.

Separately, some technical terms have multiple equivalent spoken variations and which one is best to use when and why is an unsolvable mystery. One such case is the verbal equivalent of [Big O notation](https://en.wikipedia.org/wiki/Big_O_notation) (update: what I mean here is not strictly big o notation, but the language around it for computer science context). I've done a few interviews recently, and I noticed that my interviewers used a variety of terms, and my own phrasing has awkwardly shifted around too. Maybe we can figure something out together:


<div>
<button onclick="nextWord()">How to talk about O(n²) in conversation?</button><br><br>

<p id="demo">"__________________"</p><br>
Vote: <button onclick="yesVote()">It sounds good</button> <button onclick="mehVote()">It sounds ok</button>
<button onclick="noVote()">It sounds wrong or confusing</button>


<script>
// @license magnet:?xt=urn:btih:cf05388f2679ee054f2beb29a391d25f4e673ac3&dn=gpl-2.0.txt GPLv2.0 or later
var words = [
"asymtotic bound of n squared",
"asymtotic bound n squared",
"asymtotic run time of n squared",
"asymtotic run time n squared",
"asymtotic growth of n squared",
"asymtotic growth n squared",
"asymtotic growth rate of n squared",
"asymtotic growth rate n squared",
"bound of n squared",
"bound n squared",
"run time of n squared",
"run time n squared",
"growth of n squared",
"growth n squared",
"growth rate of n squared",
"growth rate n squared",
"computational complexity of n squared",
"computational complexity n squared",
"computational efficiency of n squared",
"computational efficiency n squared",
"computational growth rate of n squared",
"computational growth rate n squared",
"big oh complexity of n squared",
"big oh complexity n squared",
"big oh efficiency of n squared",
"big oh efficiency n squared",
"big oh growth rate of n squared",
"big oh growth rate n squared",
"algorithmic complexity of n squared",
"algorithmic complexity n squared",
"algorithmic efficiency of n squared",
"algorithmic efficiency n squared",
"algorithmic growth rate of n squared",
"algorithmic growth rate n squared",
"time complexity of n squared",
"time complexity n squared",
"time efficiency of n squared",
"time efficiency n squared",
"time growth rate of n squared",
"time growth rate n squared",
"asymtotic complexity of n squared",
"asymtotic complexity n squared",
"asymtotic efficiency of n squared",
"asymtotic efficiency n squared",
"asymtotic growth rate of n squared",
"asymtotic growth rate n squared",
"complexity of n squared",
"complexity n squared",
"efficiency of n squared",
"efficiency n squared",
"growth rate of n squared",
"growth rate n squared",
"big oh of n squared",
"big oh n squared",
"order of n squared",
"order n squared",
"order of complexity of n squared",
"order of complexity n squared",
"time of n squared",
"time n squared",
"quadratic asymtotic bound",
"quadratic asymtotic run time",
"quadratic asymtotic growth",
"quadratic asymtotic growth rate",
"quadratic bound",
"quadratic run time",
"quadratic growth",
"quadratic growth rate",
"quadratic computational complexity",
"quadratic computational efficiency",
"quadratic computational growth rate",
"quadratic big oh complexity",
"quadratic big oh efficiency",
"quadratic big oh growth rate",
"quadratic algorithmic complexity",
"quadratic algorithmic efficiency",
"quadratic algorithmic growth rate",
"quadratic time complexity",
"quadratic time efficiency",
"quadratic time growth rate",
"quadratic asymtotic complexity",
"quadratic asymtotic efficiency",
"quadratic asymtotic growth rate",
"quadratic complexity",
"quadratic efficiency",
"quadratic growth rate",
"quadratic big oh",
"quadratic order",
"quadratic order of complexity",
"quadratic time",
"oh n squared",
"oh of n squared"];

function yesVote() {
    nextWord("yes")
}
function mehVote() {
    nextWord("meh")
}
function noVote() {
    nextWord("bad")
}

word = "The end."
function nextWord(vote) {
    if ('undefined' !== typeof vote) {
        var xmlhttp = new XMLHttpRequest();
        xmlhttp.open("POST","/On2voting.py",true);
        xmlhttp.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
        xmlhttp.send(JSON.stringify({vote:vote, word:word}));
    }
    word = "The end."
    if (words.length >= 1) {
        wordIndex = Math.floor(Math.random() * words.length)
        word = words[wordIndex];
    }
    document.getElementById("demo").innerHTML = '"' + word + '"';
    words.splice(wordIndex, 1)

};
// @license-end
</script>
</div>
My next blog post [has the results](/blog/on2-vote-results.html).

Where did I get these terms? Just a few clicks around the internet and my own experience. I started idly writing a few, which started to become a [Backus–Naur Form (BNF)](https://en.wikipedia.org/wiki/Backus%E2%80%93Naur_Form). Then I got quite annoyed that I couldn't figure out how to properly pronounce Naur. Turns out it's [like now-ah](http://forvo.com/search/naur/). Then I wrote a python program which expands the BNF to all possible combinations:

~~~ py
from itertools import product

def bnf_expand(*args):
    result = []
    for arg in args:
        result.extend([" ".join(filter(None, x)) for x in product(*arg)])
    return result

# the BNF in reverse of normal order
asymtotic = ['asymtotic', '']
asymtotic_modified = ['bound', 'run time', 'growth', 'growth rate']
commonly_modified = ['complexity', 'efficiency', 'growth rate']
common_modifiers = ['computational', 'big oh', 'algorithmic', 'time',
                     'asymtotic', '']
unmodified = ['big oh', 'order', 'order of complexity', 'time']
O = bnf_expand([asymtotic, asymtotic_modified],
                [common_modifiers, commonly_modified], [unmodified])
of = ['of', '']
On2 = bnf_expand([O, of, ["n squared"]], [['quadratic'], O],
                 [['oh n squared']], [['oh of n squared']])

# Print the top level expanded BNF
print On2
~~~

The result is 92 terms which can work the same in a conversation (unless your votes/comments say otherwise). And that doesn't even consider whether to say someting _is_ O(n²), _has_ O(n²), or _is of_ O(n²).

The list is a bit long, so I put them in javascript and made some buttons.

You might call all these words "terms of art", or "jargon", or "lingo"... orrr, let's call the whole thing off.
